<?php declare(strict_types=1);

namespace App\DataFixtures;

use App\Entity\MenuItem;
use App\Entity\MenuItemAlias;
use App\Entity\Vendor;
use Doctrine\Bundle\FixturesBundle\Fixture;
use Doctrine\Persistence\ObjectManager;
use Faker\Factory;
use Faker\Generator;
use Override;

use function random_int;
use function range;

class AppFixtures extends Fixture
{
    private readonly Generator $faker;

    public function __construct()
    {
        $this->faker = Factory::create();
    }

    #[Override]
    public function load(ObjectManager $manager): void
    {
        foreach (range(0, 10) as $counter) {
            $vendor = new Vendor(name: $this->faker->company());
            $this->addMenuItemsToVendor($vendor, $manager);
            $manager->persist($vendor);
        }

        $manager->flush();
    }

    private function addMenuItemsToVendor(Vendor $vendor, ObjectManager $manager): void
    {
        foreach (range(0, 100) as $counter) {
            $aliasOne = MenuItemAlias::new($this->faker->lastName());
            $aliasTwo = MenuItemAlias::new($this->faker->lastName());
            $manager->persist($aliasOne);
            $manager->persist($aliasTwo);
            $item = MenuItem::new([$aliasOne, $aliasTwo]);
            $item->price = random_int(500, 2000);
            $manager->persist($item);
            $vendor->addMenuItem($item);
        }
    }
}
