<?php declare(strict_types=1);

namespace App\Entity;

use App\Repository\FoodOrderRepository;
use DateTimeImmutable;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Bridge\Doctrine\IdGenerator\UlidGenerator;
use Symfony\Bridge\Doctrine\Types\UlidType;
use Symfony\Component\Uid\Ulid;

#[ORM\Entity(repositoryClass: FoodOrderRepository::class)]
class FoodOrder
{
    public function __construct(
        #[ORM\Column]
        public string                 $startedByName,
        #[ORM\Id]
        #[ORM\GeneratedValue(strategy: 'CUSTOM')]
        #[ORM\Column(type: UlidType::NAME, unique: true)]
        #[ORM\CustomIdGenerator(class: UlidGenerator::class)]
        public Ulid                   $id = new Ulid,
        #[ORM\Column(nullable: true)]
        public DateTimeImmutable|null $closedAt = null,

        /** @var Collection<int, OrderItem> $orderItems */
        #[ORM\OneToMany(
            mappedBy: 'foodOrder',
            targetEntity: OrderItem::class,
            orphanRemoval: true,
        )]
        public Collection             $orderItems = new ArrayCollection,
        #[ORM\ManyToOne(inversedBy: 'foodOrders')]
        #[ORM\JoinColumn(nullable: false)]
        public Vendor|null            $vendor = null,
        #[ORM\Column]
        public DateTimeImmutable      $startedAt = new DateTimeImmutable,
    )
    {
    }

    public function addOrderItem(OrderItem $orderItem): static
    {
        if (!$this->orderItems->contains($orderItem)) {
            $this->orderItems->add($orderItem);
            $orderItem->foodOrder = $this;
        }

        return $this;
    }

    public function removeOrderItem(OrderItem $orderItem): static
    {
        // set the owning side to null (unless already changed)
        if (
            $this->orderItems->removeElement($orderItem)
            && $orderItem->foodOrder === $this
        ) {
            $orderItem->foodOrder = null;
        }

        return $this;
    }

    public function groupedOrderItems(): array
    {
        return $this->orderItems->reduce(
            function (array $carry, OrderItem $item): array {
                $menuItemStringId = (string)$item->menuItem->id;
                if (isset($carry[$menuItemStringId])) {
                    $carry[$menuItemStringId]['amount']++;
                } else {
                    $carry[$menuItemStringId] = [
                        'item' => $item,
                        'amount' => 1,
                    ];
                }
                return $carry;
            },
            []
        );
    }
}
